module.exports = {
  "env": {
    "browser": true,
    "es2018": true,
    "node": true
  },
  "extends": "eslint:recommended",
  "parserOptions": {
    "ecmaVersion": 2018,
    "sourceType": "module"
  },
  "rules": {
    "no-cond-assign": 0
  }
}
